@@ -22,30 +22,16 @@
 namespace mozilla {
 namespace dom {
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(AudioContext)
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_NATIVE(AudioContext)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mWindow)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mDestination)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mListener)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER_NATIVE
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_BEGIN(AudioContext)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mWindow)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDestination)
-  // Cannot use NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR since AudioListener
-  // does not inherit from nsISupports.
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mListener)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_SCRIPT_OBJECTS
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-NS_IMPL_CYCLE_COLLECTION_TRACE_NATIVE_BEGIN(AudioContext)
-  NS_IMPL_CYCLE_COLLECTION_TRACE_PRESERVED_WRAPPER
-NS_IMPL_CYCLE_COLLECTION_TRACE_END
+NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_3(AudioContext,
+                                        mWindow, mDestination, mListener)
+
 NS_IMPL_CYCLE_COLLECTION_ROOT_NATIVE(AudioContext, AddRef)
 NS_IMPL_CYCLE_COLLECTION_UNROOT_NATIVE(AudioContext, Release)
 
 AudioContext::AudioContext(nsIDOMWindow* aWindow)
   : mWindow(aWindow)
   , mDestination(new AudioDestinationNode(this))
+  , mSampleRate(44100) // hard-code for now
 {
   SetIsDOMBinding();
 }
@@ -64,7 +50,7 @@ AudioContext::WrapObject(JSContext* aCx, JSObject* aScope,
 /* static */ already_AddRefed<AudioContext>
 AudioContext::Constructor(nsISupports* aGlobal, ErrorResult& aRv)
 {
-  nsCOMPtr<nsIDOMWindow> window = do_QueryInterface(aGlobal);
+  nsCOMPtr<nsPIDOMWindow> window = do_QueryInterface(aGlobal);
   if (!window) {
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
@@ -72,6 +58,7 @@ AudioContext::Constructor(nsISupports* aGlobal, ErrorResult& aRv)
 
   AudioContext* object = new AudioContext(window);
   NS_ADDREF(object);
+  window->AddAudioContext(object);
   return object;
 }
 
@@ -104,9 +91,9 @@ AudioContext::CreateGain()
 }
 
 already_AddRefed<DelayNode>
-AudioContext::CreateDelay(float aMaxDelayTime, ErrorResult& aRv)
+AudioContext::CreateDelay(double aMaxDelayTime, ErrorResult& aRv)
 {
-  if (aMaxDelayTime > 0.f && aMaxDelayTime < 3.f) {
+  if (aMaxDelayTime > 0. && aMaxDelayTime < 3.) {
     nsRefPtr<DelayNode> delayNode = new DelayNode(this, aMaxDelayTime);
     return delayNode.forget();
   }
