@@ -8,115 +8,19 @@
 #include "nsSVGAngle.h"
 #include "prdtoa.h"
 #include "nsTextFormatter.h"
+#include "nsSVGAttrTearoffTable.h"
 #include "nsSVGMarkerElement.h"
 #include "nsMathUtils.h"
 #include "nsContentUtils.h" // NS_ENSURE_FINITE
 #include "nsSMILValue.h"
 #include "SVGOrientSMILType.h"
 #include "nsAttrValueInlines.h"
+#include "SVGAngle.h"
+#include "SVGAnimatedAngle.h"
 #include "mozilla/Attributes.h"
 
 using namespace mozilla;
-
-/**
- * Mutable SVGAngle class for SVGSVGElement.createSVGAngle().
- *
- * Note that this class holds its own nsSVGAngle, which therefore can't be
- * animated. This means SVGMarkerElement::setOrientToAngle(angle) must copy
- * any DOMSVGAngle passed in. Perhaps this is wrong and inconsistent with
- * other parts of SVG, but it's how the code works for now.
- */
-class DOMSVGAngle MOZ_FINAL : public nsIDOMSVGAngle
-{
-public:
-  NS_DECL_ISUPPORTS
-
-  DOMSVGAngle()
-    { mVal.Init(); }
-    
-  NS_IMETHOD GetUnitType(uint16_t* aResult)
-    { *aResult = mVal.mBaseValUnit; return NS_OK; }
-
-  NS_IMETHOD GetValue(float* aResult)
-    { *aResult = mVal.GetBaseValue(); return NS_OK; }
-  NS_IMETHOD SetValue(float aValue)
-    {
-      NS_ENSURE_FINITE(aValue, NS_ERROR_ILLEGAL_VALUE);
-      mVal.SetBaseValue(aValue, nullptr, false);
-      return NS_OK;
-    }
-
-  NS_IMETHOD GetValueInSpecifiedUnits(float* aResult)
-    { *aResult = mVal.mBaseVal; return NS_OK; }
-  NS_IMETHOD SetValueInSpecifiedUnits(float aValue)
-    {
-      NS_ENSURE_FINITE(aValue, NS_ERROR_ILLEGAL_VALUE);
-      mVal.mBaseVal = aValue;
-      return NS_OK;
-    }
-
-  NS_IMETHOD SetValueAsString(const nsAString& aValue)
-    { return mVal.SetBaseValueString(aValue, nullptr, false); }
-  NS_IMETHOD GetValueAsString(nsAString& aValue)
-    { mVal.GetBaseValueString(aValue); return NS_OK; }
-
-  NS_IMETHOD NewValueSpecifiedUnits(uint16_t unitType,
-                                    float valueInSpecifiedUnits)
-    {
-      return mVal.NewValueSpecifiedUnits(unitType, valueInSpecifiedUnits, nullptr);
-    }
-
-  NS_IMETHOD ConvertToSpecifiedUnits(uint16_t unitType)
-    { return mVal.ConvertToSpecifiedUnits(unitType, nullptr); }
-
-private:
-  nsSVGAngle mVal;
-};
-
-NS_SVG_VAL_IMPL_CYCLE_COLLECTION(nsSVGAngle::DOMBaseVal, mSVGElement)
-
-NS_SVG_VAL_IMPL_CYCLE_COLLECTION(nsSVGAngle::DOMAnimVal, mSVGElement)
-
-NS_SVG_VAL_IMPL_CYCLE_COLLECTION(nsSVGAngle::DOMAnimatedAngle, mSVGElement)
-
-NS_IMPL_CYCLE_COLLECTING_ADDREF(nsSVGAngle::DOMBaseVal)
-NS_IMPL_CYCLE_COLLECTING_RELEASE(nsSVGAngle::DOMBaseVal)
-
-NS_IMPL_CYCLE_COLLECTING_ADDREF(nsSVGAngle::DOMAnimVal)
-NS_IMPL_CYCLE_COLLECTING_RELEASE(nsSVGAngle::DOMAnimVal)
-
-NS_IMPL_CYCLE_COLLECTING_ADDREF(nsSVGAngle::DOMAnimatedAngle)
-NS_IMPL_CYCLE_COLLECTING_RELEASE(nsSVGAngle::DOMAnimatedAngle)
-
-NS_IMPL_ADDREF(DOMSVGAngle)
-NS_IMPL_RELEASE(DOMSVGAngle)
-
-DOMCI_DATA(SVGAngle, nsSVGAngle::DOMBaseVal)
-DOMCI_DATA(SVGAnimatedAngle, nsSVGAngle::DOMAnimatedAngle)
-
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsSVGAngle::DOMBaseVal)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMSVGAngle)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGAngle)
-NS_INTERFACE_MAP_END
-
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsSVGAngle::DOMAnimVal)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMSVGAngle)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGAngle)
-NS_INTERFACE_MAP_END
-
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsSVGAngle::DOMAnimatedAngle)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMSVGAnimatedAngle)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGAnimatedAngle)
-NS_INTERFACE_MAP_END
-
-NS_INTERFACE_MAP_BEGIN(DOMSVGAngle)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMSVGAngle)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGAngle)
-NS_INTERFACE_MAP_END
+using namespace mozilla::dom;
 
 static nsIAtom** const unitMap[] =
 {
@@ -127,13 +31,20 @@ static nsIAtom** const unitMap[] =
   &nsGkAtoms::grad
 };
 
+static nsSVGAttrTearoffTable<nsSVGAngle, SVGAnimatedAngle>
+  sSVGAnimatedAngleTearoffTable;
+static nsSVGAttrTearoffTable<nsSVGAngle, SVGAngle>
+  sBaseSVGAngleTearoffTable;
+static nsSVGAttrTearoffTable<nsSVGAngle, SVGAngle>
+  sAnimSVGAngleTearoffTable;
+
 /* Helper functions */
 
 static bool
 IsValidUnitType(uint16_t unit)
 {
-  if (unit > nsIDOMSVGAngle::SVG_ANGLETYPE_UNKNOWN &&
-      unit <= nsIDOMSVGAngle::SVG_ANGLETYPE_GRAD)
+  if (unit > SVG_ANGLETYPE_UNKNOWN &&
+      unit <= SVG_ANGLETYPE_GRAD)
     return true;
 
   return false;
@@ -157,7 +68,7 @@ static uint16_t
 GetUnitTypeForString(const nsAString& unitStr)
 {
   if (unitStr.IsEmpty()) 
-    return nsIDOMSVGAngle::SVG_ANGLETYPE_UNSPECIFIED;
+    return SVG_ANGLETYPE_UNSPECIFIED;
                    
   nsIAtom *unitAtom = NS_GetStaticAtom(unitStr);
 
@@ -169,7 +80,7 @@ GetUnitTypeForString(const nsAString& unitStr)
     }
   }
 
-  return nsIDOMSVGAngle::SVG_ANGLETYPE_UNKNOWN;
+  return SVG_ANGLETYPE_UNKNOWN;
 }
 
 static void
@@ -214,12 +125,12 @@ GetValueFromString(const nsAString &aValueAsString,
 nsSVGAngle::GetDegreesPerUnit(uint8_t aUnit)
 {
   switch (aUnit) {
-  case nsIDOMSVGAngle::SVG_ANGLETYPE_UNSPECIFIED:
-  case nsIDOMSVGAngle::SVG_ANGLETYPE_DEG:
+  case SVG_ANGLETYPE_UNSPECIFIED:
+  case SVG_ANGLETYPE_DEG:
     return 1;
-  case nsIDOMSVGAngle::SVG_ANGLETYPE_RAD:
+  case SVG_ANGLETYPE_RAD:
     return static_cast<float>(180.0 / M_PI);
-  case nsIDOMSVGAngle::SVG_ANGLETYPE_GRAD:
+  case SVG_ANGLETYPE_GRAD:
     return 90.0f / 100.0f;
   default:
     NS_NOTREACHED("Unknown unit type");
@@ -307,27 +218,44 @@ nsSVGAngle::NewValueSpecifiedUnits(uint16_t unitType,
 }
 
 nsresult
-nsSVGAngle::ToDOMBaseVal(nsIDOMSVGAngle **aResult, nsSVGElement *aSVGElement)
+nsSVGAngle::ToDOMBaseVal(SVGAngle **aResult, nsSVGElement *aSVGElement)
 {
-  *aResult = new DOMBaseVal(this, aSVGElement);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsRefPtr<SVGAngle> domBaseVal =
+    sBaseSVGAngleTearoffTable.GetTearoff(this);
+  if (!domBaseVal) {
+    domBaseVal = new SVGAngle(this, aSVGElement, SVGAngle::BaseValue);
+    sBaseSVGAngleTearoffTable.AddTearoff(this, domBaseVal);
+  }
 
-  NS_ADDREF(*aResult);
+  domBaseVal.forget(aResult);
   return NS_OK;
 }
 
 nsresult
-nsSVGAngle::ToDOMAnimVal(nsIDOMSVGAngle **aResult, nsSVGElement *aSVGElement)
+nsSVGAngle::ToDOMAnimVal(SVGAngle **aResult, nsSVGElement *aSVGElement)
 {
-  *aResult = new DOMAnimVal(this, aSVGElement);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsRefPtr<SVGAngle> domAnimVal =
+    sAnimSVGAngleTearoffTable.GetTearoff(this);
+  if (!domAnimVal) {
+    domAnimVal = new SVGAngle(this, aSVGElement, SVGAngle::AnimValue);
+    sAnimSVGAngleTearoffTable.AddTearoff(this, domAnimVal);
+  }
 
-  NS_ADDREF(*aResult);
+  domAnimVal.forget(aResult);
   return NS_OK;
 }
 
+SVGAngle::~SVGAngle()
+{
+  if (mType == BaseValue) {
+    sBaseSVGAngleTearoffTable.RemoveTearoff(mVal);
+  } else if (mType == AnimValue) {
+    sAnimSVGAngleTearoffTable.RemoveTearoff(mVal);
+  } else {
+    delete mVal;
+  }
+}
+
 /* Implementation */
 
 nsresult
@@ -415,26 +343,23 @@ nsSVGAngle::SetAnimValue(float aValue, uint8_t aUnit, nsSVGElement *aSVGElement)
 }
 
 nsresult
-nsSVGAngle::ToDOMAnimatedAngle(nsIDOMSVGAnimatedAngle **aResult,
+nsSVGAngle::ToDOMAnimatedAngle(nsISupports **aResult,
                                nsSVGElement *aSVGElement)
 {
-  *aResult = new DOMAnimatedAngle(this, aSVGElement);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsRefPtr<SVGAnimatedAngle> domAnimatedAngle =
+    sSVGAnimatedAngleTearoffTable.GetTearoff(this);
+  if (!domAnimatedAngle) {
+    domAnimatedAngle = new SVGAnimatedAngle(this, aSVGElement);
+    sSVGAnimatedAngleTearoffTable.AddTearoff(this, domAnimatedAngle);
+  }
 
-  NS_ADDREF(*aResult);
+  domAnimatedAngle.forget(aResult);
   return NS_OK;
 }
 
-nsresult
-NS_NewDOMSVGAngle(nsIDOMSVGAngle** aResult)
+SVGAnimatedAngle::~SVGAnimatedAngle()
 {
-  *aResult = new DOMSVGAngle;
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_ADDREF(*aResult);
-  return NS_OK;
+  sSVGAnimatedAngleTearoffTable.RemoveTearoff(mVal);
 }
 
 nsISMILAttr*
@@ -507,7 +432,7 @@ nsSVGAngle::SMILOrient::SetAnimValue(const nsSMILValue& aValue)
   if (aValue.mType == &SVGOrientSMILType::sSingleton) {
     mOrientType->SetAnimValue(aValue.mU.mOrient.mOrientType);
     if (aValue.mU.mOrient.mOrientType == nsIDOMSVGMarkerElement::SVG_MARKER_ORIENT_AUTO) {
-      mAngle->SetAnimValue(0.0f, nsIDOMSVGAngle::SVG_ANGLETYPE_UNSPECIFIED, mSVGElement);
+      mAngle->SetAnimValue(0.0f, SVG_ANGLETYPE_UNSPECIFIED, mSVGElement);
     } else {
       mAngle->SetAnimValue(aValue.mU.mOrient.mAngle, aValue.mU.mOrient.mUnit, mSVGElement);
     }
