@@ -8,6 +8,7 @@
 
 #include "AsyncConnectionHelper.h"
 
+#include "mozilla/dom/quota/QuotaManager.h"
 #include "mozilla/storage.h"
 #include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
@@ -24,6 +25,7 @@
 #include "ipc/IndexedDBParent.h"
 
 USING_INDEXEDDB_NAMESPACE
+using mozilla::dom::quota::QuotaManager;
 
 namespace {
 
@@ -287,7 +289,7 @@ AsyncConnectionHelper::Run()
   if (NS_SUCCEEDED(rv)) {
     bool hasSavepoint = false;
     if (mDatabase) {
-      IndexedDatabaseManager::SetCurrentWindow(mDatabase->GetOwner());
+      QuotaManager::SetCurrentWindow(mDatabase->GetOwner());
 
       // Make the first savepoint.
       if (mTransaction) {
@@ -313,7 +315,7 @@ AsyncConnectionHelper::Run()
 
       // Don't unset this until we're sure that all SQLite activity has
       // completed!
-      IndexedDatabaseManager::SetCurrentWindow(nullptr);
+      QuotaManager::SetCurrentWindow(nullptr);
     }
   }
   else {
@@ -449,7 +451,7 @@ AsyncConnectionHelper::OnSuccess()
                mTransaction->IsAborted(),
                "How else can this be closed?!");
 
-  if ((internalEvent->flags & NS_EVENT_FLAG_EXCEPTION_THROWN) &&
+  if (internalEvent->mFlags.mExceptionHasBeenRisen &&
       mTransaction &&
       mTransaction->IsOpen()) {
     rv = mTransaction->Abort(NS_ERROR_DOM_INDEXEDDB_ABORT_ERR);
@@ -485,7 +487,7 @@ AsyncConnectionHelper::OnError()
     nsEvent* internalEvent = event->GetInternalNSEvent();
     NS_ASSERTION(internalEvent, "This should never be null!");
 
-    if ((internalEvent->flags & NS_EVENT_FLAG_EXCEPTION_THROWN) &&
+    if (internalEvent->mFlags.mExceptionHasBeenRisen &&
         mTransaction &&
         mTransaction->IsOpen() &&
         NS_FAILED(mTransaction->Abort(NS_ERROR_DOM_INDEXEDDB_ABORT_ERR))) {
