@@ -11,17 +11,15 @@ import subprocess
 from subprocess import *
 from threading import Thread
 
-DEBUGGER_INFO = {
-  "gdb": {
-    "interactive": True,
-    "args": "-q --args"
-  },
+def add_libdir_to_path():
+    from os.path import dirname, exists, join, realpath
+    js_src_dir = dirname(dirname(realpath(sys.argv[0])))
+    assert exists(join(js_src_dir,'jsapi.h'))
+    sys.path.append(join(js_src_dir, 'lib'))
+    sys.path.append(join(js_src_dir, 'tests', 'lib'))
 
-  "valgrind": {
-    "interactive": False,
-    "args": "--leak-check=full"
-  }
-}
+add_libdir_to_path()
+from progressbar import ProgressBar, NullProgressBar
 
 # Backported from Python 3.1 posixpath.py
 def _relpath(path, start=None):
@@ -49,7 +47,7 @@ os.path.relpath = _relpath
 class Test:
     def __init__(self, path):
         self.path = path       # path to test file
-        
+
         self.jitflags = []     # jit flags to enable
         self.slow = False      # True means the test is slow-running
         self.allow_oom = False # True means that OOM is not considered a failure
@@ -204,7 +202,7 @@ def run_cmd(cmdline, env, timeout):
 def run_cmd_avoid_stdio(cmdline, env, timeout):
     stdoutPath, stderrPath = tmppath('jsstdout'), tmppath('jsstderr')
     env['JS_STDOUT'] = stdoutPath
-    env['JS_STDERR'] = stderrPath       
+    env['JS_STDERR'] = stderrPath
     _, __, code = run_timeout_cmd(cmdline, { 'env': env }, timeout)
     return read_and_unlink(stdoutPath), read_and_unlink(stderrPath), code
 
@@ -216,8 +214,10 @@ def run_test(test, lib_dir, shell_args):
              for d in os.environ['PATH'].split(os.pathsep)])):
         valgrind_prefix = [ 'valgrind',
                             '-q',
-                            '--smc-check=all',
+                            '--smc-check=all-non-file',
                             '--error-exitcode=1',
+                            '--gen-suppressions=all',
+                            '--show-possibly-lost=no',
                             '--leak-check=full']
         if os.uname()[0] == 'Darwin':
             valgrind_prefix += ['--dsymutil=yes']
@@ -273,13 +273,15 @@ def print_tinderbox(label, test, message=None):
     print result
 
 def run_tests(tests, test_dir, lib_dir, shell_args):
-    pb = None
-    if not OPTIONS.hide_progress and not OPTIONS.show_cmd:
-        try:
-            from progressbar import ProgressBar
-            pb = ProgressBar('', len(tests), 24)
-        except ImportError:
-            pass
+    pb = NullProgressBar()
+    if not OPTIONS.hide_progress and not OPTIONS.show_cmd and ProgressBar.conservative_isatty():
+        fmt = [
+            {'value': 'PASS',    'color': 'green'},
+            {'value': 'FAIL',    'color': 'red'},
+            {'value': 'TIMEOUT', 'color': 'blue'},
+            {'value': 'SKIP',    'color': 'brightgray'},
+        ]
+        pb = ProgressBar(len(tests), fmt)
 
     failures = []
     timeouts = 0
@@ -293,6 +295,7 @@ def run_tests(tests, test_dir, lib_dir, shell_args):
 
             if not ok:
                 failures.append([ test, out, err, code, timed_out ])
+                pb.message("FAIL - %s" % test.path)
             if timed_out:
                 timeouts += 1
 
@@ -309,15 +312,17 @@ def run_tests(tests, test_dir, lib_dir, shell_args):
                     print_tinderbox("TEST-UNEXPECTED-FAIL", test, msg);
 
             n = i + 1
-            if pb:
-                pb.label = '[%4d|%4d|%4d|%4d]'%(n - len(failures), len(failures), timeouts, n)
-                pb.update(n)
+            pb.update(n, {
+                'PASS': n - len(failures),
+                'FAIL': len(failures),
+                'TIMEOUT': timeouts,
+                'SKIP': 0}
+            )
         complete = True
     except KeyboardInterrupt:
         print_tinderbox("TEST-UNEXPECTED-FAIL", test);
 
-    if pb:
-        pb.finish()
+    pb.finish(True)
 
     if failures:
         if OPTIONS.write_failures:
@@ -362,7 +367,7 @@ def run_tests(tests, test_dir, lib_dir, shell_args):
         return True
 
 def parse_jitflags():
-    jitflags = [ [ '-' + flag for flag in flags ] 
+    jitflags = [ [ '-' + flag for flag in flags ]
                  for flags in OPTIONS.jitflags.split(',') ]
     for flags in jitflags:
         for flag in flags:
@@ -402,7 +407,7 @@ def main(argv):
     op = OptionParser(usage='%prog [options] JS_SHELL [TESTS]')
     op.add_option('-s', '--show-cmd', dest='show_cmd', action='store_true',
                   help='show js shell command run')
-    op.add_option('-f', '--show-failed-cmd', dest='show_failed', 
+    op.add_option('-f', '--show-failed-cmd', dest='show_failed',
                   action='store_true', help='show command lines of failed tests')
     op.add_option('-o', '--show-output', dest='show_output', action='store_true',
                   help='show output from js shell')
@@ -430,14 +435,15 @@ def main(argv):
                   help='Enable the |valgrind| flag, if valgrind is in $PATH.')
     op.add_option('--valgrind-all', dest='valgrind_all', action='store_true',
                   help='Run all tests with valgrind, if valgrind is in $PATH.')
-    op.add_option('--jitflags', dest='jitflags', default='m,mn',
-                  help='Example: --jitflags=m,mn to run each test with -m, -m -n [default=%default]')
+    op.add_option('--jitflags', dest='jitflags', default='',
+                  help='Example: --jitflags=m,mn to run each test with "-m" and "-m -n" [default="%default"]. ' +
+                       'Long flags, such as "--no-jm", should be set using --args.')
     op.add_option('--avoid-stdio', dest='avoid_stdio', action='store_true',
                   help='Use js-shell file indirection instead of piping stdio.')
     op.add_option('--write-failure-output', dest='write_failure_output', action='store_true',
                   help='With --write-failures=FILE, additionally write the output of failed tests to [FILE]')
     op.add_option('--ion', dest='ion', action='store_true',
-                  help='Run tests with --ion flag (ignores --jitflags)')
+                  help='Run tests once with --ion-eager and once with --no-jm (ignores --jitflags)')
     op.add_option('--tbpl', dest='tbpl', action='store_true',
                   help='Run tests with all IonMonkey option combinations (ignores --jitflags)')
     (OPTIONS, args) = op.parse_args(argv)
