@@ -5,6 +5,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/DebugOnly.h"
+
 #include "jsanalyze.h"
 #include "jscntxt.h"
 #include "jsscope.h"
@@ -36,24 +38,28 @@
 
 #include "jsautooplen.h"
 
+#include "ion/Ion.h"
+
 using namespace js;
 using namespace js::mjit;
 using namespace JSC;
 
+using mozilla::DebugOnly;
+
 using ic::Repatcher;
 
 static jsbytecode *
 FindExceptionHandler(JSContext *cx)
 {
     StackFrame *fp = cx->fp();
-    JSScript *script = fp->script();
+    RootedScript script(cx, fp->script());
 
     if (!script->hasTrynotes())
         return NULL;
 
   error:
     if (cx->isExceptionPending()) {
-        for (TryNoteIter tni(cx->regs()); !tni.done(); ++tni) {
+        for (TryNoteIter tni(cx, cx->regs()); !tni.done(); ++tni) {
             JSTryNote *tn = *tni;
 
             UnwindScope(cx, tn->stackDepth);
@@ -105,7 +111,8 @@ FindExceptionHandler(JSContext *cx)
                    * pending exception.
                    */
                   JS_ASSERT(JSOp(*pc) == JSOP_ENDITER);
-                  bool ok = UnwindIteratorForException(cx, &cx->regs().sp[-1].toObject());
+                  RootedObject obj(cx, &cx->regs().sp[-1].toObject());
+                  bool ok = UnwindIteratorForException(cx, obj);
                   cx->regs().sp -= 1;
                   if (!ok)
                       goto error;
@@ -133,7 +140,8 @@ stubs::SlowCall(VMFrame &f, uint32_t argc)
     if (!InvokeKernel(f.cx, args))
         THROW();
 
-    types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+    RootedScript fscript(f.cx, f.script());
+    types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
 }
 
 void JS_FASTCALL
@@ -143,7 +151,8 @@ stubs::SlowNew(VMFrame &f, uint32_t argc)
     if (!InvokeConstructorKernel(f.cx, args))
         THROW();
 
-    types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+    RootedScript fscript(f.cx, f.script());
+    types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
 }
 
 static inline bool
@@ -180,6 +189,7 @@ stubs::HitStackQuota(VMFrame &f)
 void * JS_FASTCALL
 stubs::FixupArity(VMFrame &f, uint32_t nactual)
 {
+    AssertCanGC();
     JSContext *cx = f.cx;
     StackFrame *oldfp = f.fp();
 
@@ -191,9 +201,9 @@ stubs::FixupArity(VMFrame &f, uint32_t nactual)
      * members that have been initialized by the caller and early prologue.
      */
     InitialFrameFlags initial = oldfp->initialFlags();
-    JSFunction *fun           = oldfp->fun();
-    JSScript *script          = fun->script();
-    void *ncode               = oldfp->nativeReturnAddress();
+    RootedFunction fun(cx, oldfp->fun());
+    RootedScript script(cx, fun->nonLazyScript());
+    void *ncode = oldfp->nativeReturnAddress();
 
     /* Pop the inline frame. */
     f.regs.popPartialFrame((Value *)oldfp);
@@ -241,32 +251,66 @@ stubs::CompileFunction(VMFrame &f, uint32_t argc)
         return UncachedCall(f, argc);
 }
 
+// Heuristics to decide whether a JM function call should invoke JM or Ion. Calling
+// into Ion may be faster, especially if the function contains loops, but JM -> Ion
+// calls are slower than JM -> JM calls.
+static inline bool
+ShouldJaegerCompileCallee(JSContext *cx, JSScript *caller, JSScript *callee, JITScript *callerJit)
+{
+#ifdef JS_ION
+    if (!ion::IsEnabled(cx))
+        return true;
+
+    // If we know Ion cannot compile either the caller or callee, use JM.
+    if (!callee->canIonCompile())
+        return true;
+
+    // Use JM if the callee has no loops. In this case calling into Ion
+    // is likely not worth the overhead.
+    if (!callee->hasAnalysis())
+        return true;
+
+    if (callee->isShortRunning())
+        return true;
+
+    return false;
+#endif
+    return true;
+}
+
 static inline bool
 UncachedInlineCall(VMFrame &f, InitialFrameFlags initial,
                    void **pret, bool *unjittable, uint32_t argc)
 {
+    AssertCanGC();
     JSContext *cx = f.cx;
     CallArgs args = CallArgsFromSp(argc, f.regs.sp);
-    JSFunction *newfun = args.callee().toFunction();
-    JSScript *newscript = newfun->script();
+    RootedFunction newfun(cx, args.callee().toFunction());
+
+    RootedScript newscript(cx, newfun->getOrCreateScript(cx));
+    if (!newscript)
+        return false;
 
     bool construct = InitialFrameFlagsAreConstructing(initial);
 
+    RootedScript fscript(cx, f.script());
     bool newType = construct && cx->typeInferenceEnabled() &&
-        types::UseNewType(cx, f.script(), f.pc());
+        types::UseNewType(cx, fscript, f.pc());
 
     if (!types::TypeMonitorCall(cx, args, construct))
         return false;
 
     /* Try to compile if not already compiled. */
-    CompileStatus status = CanMethodJIT(cx, newscript, newscript->code, construct,
-                                        CompileRequest_Interpreter, f.fp());
-    if (status == Compile_Error) {
-        /* A runtime exception was thrown, get out. */
-        return false;
+    if (ShouldJaegerCompileCallee(cx, f.script(), newscript, f.jit())) {
+        CompileStatus status = CanMethodJIT(cx, newscript, newscript->code, construct,
+                                            CompileRequest_JIT, f.fp());
+        if (status == Compile_Error) {
+            /* A runtime exception was thrown, get out. */
+            return false;
+        }
+        if (status == Compile_Abort)
+            *unjittable = true;
     }
-    if (status == Compile_Abort)
-        *unjittable = true;
 
     /*
      * Make sure we are not calling from an inline frame if we need to make a
@@ -299,7 +343,7 @@ UncachedInlineCall(VMFrame &f, InitialFrameFlags initial,
      * will be constructing a new type object for 'this'.
      */
     if (!newType) {
-        if (JITScript *jit = newscript->getJIT(regs.fp()->isConstructing(), cx->compartment->needsBarrier())) {
+        if (JITScript *jit = newscript->getJIT(regs.fp()->isConstructing(), cx->compartment->compileBarriers())) {
             if (jit->invokeEntry) {
                 *pret = jit->invokeEntry;
 
@@ -324,11 +368,14 @@ UncachedInlineCall(VMFrame &f, InitialFrameFlags initial,
 
     JS_CHECK_RECURSION(cx, return false);
 
-    bool ok = Interpret(cx, cx->fp());
+    RootedScript script(cx, newscript);
+    bool ok = RunScript(cx, script, cx->fp());
     f.cx->stack.popInlineFrame(regs);
 
-    if (ok)
-        types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+    if (ok) {
+        RootedScript fscript(cx, f.script());
+        types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
+    }
 
     *pret = NULL;
     return ok;
@@ -337,42 +384,43 @@ UncachedInlineCall(VMFrame &f, InitialFrameFlags initial,
 void * JS_FASTCALL
 stubs::UncachedNew(VMFrame &f, uint32_t argc)
 {
-    UncachedCallResult ucr;
-    UncachedNewHelper(f, argc, &ucr);
+    UncachedCallResult ucr(f.cx);
+    UncachedNewHelper(f, argc, ucr);
     return ucr.codeAddr;
 }
 
 void
-stubs::UncachedNewHelper(VMFrame &f, uint32_t argc, UncachedCallResult *ucr)
+stubs::UncachedNewHelper(VMFrame &f, uint32_t argc, UncachedCallResult &ucr)
 {
-    ucr->init();
+    ucr.init();
     JSContext *cx = f.cx;
     CallArgs args = CallArgsFromSp(argc, f.regs.sp);
 
     /* Try to do a fast inline call before the general Invoke path. */
-    if (IsFunctionObject(args.calleev(), &ucr->fun) && ucr->fun->isInterpretedConstructor()) {
-        if (!UncachedInlineCall(f, INITIAL_CONSTRUCT, &ucr->codeAddr, &ucr->unjittable, argc))
+    if (IsFunctionObject(args.calleev(), ucr.fun.address()) && ucr.fun->isInterpretedConstructor()) {
+        if (!UncachedInlineCall(f, INITIAL_CONSTRUCT, &ucr.codeAddr, &ucr.unjittable, argc))
             THROW();
     } else {
         if (!InvokeConstructorKernel(cx, args))
             THROW();
-        types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+        RootedScript fscript(cx, f.script());
+        types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
     }
 }
 
 void * JS_FASTCALL
 stubs::UncachedCall(VMFrame &f, uint32_t argc)
 {
-    UncachedCallResult ucr;
-    UncachedCallHelper(f, argc, false, &ucr);
+    UncachedCallResult ucr(f.cx);
+    UncachedCallHelper(f, argc, false, ucr);
     return ucr.codeAddr;
 }
 
 void * JS_FASTCALL
 stubs::UncachedLoweredCall(VMFrame &f, uint32_t argc)
 {
-    UncachedCallResult ucr;
-    UncachedCallHelper(f, argc, true, &ucr);
+    UncachedCallResult ucr(f.cx);
+    UncachedCallHelper(f, argc, true, ucr);
     return ucr.codeAddr;
 }
 
@@ -385,7 +433,8 @@ stubs::Eval(VMFrame &f, uint32_t argc)
         if (!InvokeKernel(f.cx, args))
             THROW();
 
-        types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+        RootedScript fscript(f.cx, f.script());
+        types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
         return;
     }
 
@@ -393,29 +442,31 @@ stubs::Eval(VMFrame &f, uint32_t argc)
     if (!DirectEval(f.cx, args))
         THROW();
 
-    types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+    RootedScript fscript(f.cx, f.script());
+    types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
 }
 
 void
-stubs::UncachedCallHelper(VMFrame &f, uint32_t argc, bool lowered, UncachedCallResult *ucr)
+stubs::UncachedCallHelper(VMFrame &f, uint32_t argc, bool lowered, UncachedCallResult &ucr)
 {
-    ucr->init();
+    ucr.init();
 
     JSContext *cx = f.cx;
     CallArgs args = CallArgsFromSp(argc, f.regs.sp);
 
-    if (IsFunctionObject(args.calleev(), &ucr->fun)) {
-        if (ucr->fun->isInterpreted()) {
+    if (IsFunctionObject(args.calleev(), ucr.fun.address())) {
+        if (ucr.fun->isInterpreted()) {
             InitialFrameFlags initial = lowered ? INITIAL_LOWERED : INITIAL_NONE;
-            if (!UncachedInlineCall(f, initial, &ucr->codeAddr, &ucr->unjittable, argc))
+            if (!UncachedInlineCall(f, initial, &ucr.codeAddr, &ucr.unjittable, argc))
                 THROW();
             return;
         }
 
-        if (ucr->fun->isNative()) {
-            if (!CallJSNative(cx, ucr->fun->native(), args))
+        if (ucr.fun->isNative()) {
+            if (!CallJSNative(cx, ucr.fun->native(), args))
                 THROW();
-            types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+            RootedScript fscript(cx, f.script());
+            types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
             return;
         }
     }
@@ -423,7 +474,8 @@ stubs::UncachedCallHelper(VMFrame &f, uint32_t argc, bool lowered, UncachedCallR
     if (!InvokeKernel(f.cx, args))
         THROW();
 
-    types::TypeScript::Monitor(f.cx, f.script(), f.pc(), args.rval());
+    RootedScript fscript(cx, f.script());
+    types::TypeScript::Monitor(f.cx, fscript, f.pc(), args.rval());
     return;
 }
 
@@ -478,7 +530,8 @@ js_InternalThrow(VMFrame &f)
                 Value rval;
                 JSTrapStatus st = Debugger::onExceptionUnwind(cx, &rval);
                 if (st == JSTRAP_CONTINUE && handler) {
-                    st = handler(cx, cx->fp()->script(), cx->regs().pc, &rval,
+                    RootedScript fscript(cx, cx->fp()->script());
+                    st = handler(cx, fscript, cx->regs().pc, &rval,
                                  cx->runtime->debugHooks.throwHookData);
                 }
 
@@ -551,7 +604,7 @@ js_InternalThrow(VMFrame &f)
         return NULL;
 
     StackFrame *fp = cx->fp();
-    JSScript *script = fp->script();
+    RootedScript script(cx, fp->script());
 
     /*
      * Fall back to EnterMethodJIT and finish the frame in the interpreter.
@@ -562,7 +615,7 @@ js_InternalThrow(VMFrame &f)
      */
     cx->jaegerRuntime().setLastUnfinished(Jaeger_Unfinished);
 
-    if (!script->ensureRanAnalysis(cx)) {
+    if (!JSScript::ensureRanAnalysis(cx, script)) {
         js_ReportOutOfMemory(cx);
         return NULL;
     }
@@ -611,6 +664,7 @@ stubs::CreateThis(VMFrame &f, JSObject *proto)
 void JS_FASTCALL
 stubs::ScriptDebugPrologue(VMFrame &f)
 {
+    AssertCanGC();
     Probes::enterScript(f.cx, f.script(), f.script()->function(), f.fp());
     JSTrapStatus status = js::ScriptDebugPrologue(f.cx, f.fp());
     switch (status) {
@@ -637,27 +691,31 @@ stubs::ScriptDebugEpilogue(VMFrame &f)
 void JS_FASTCALL
 stubs::ScriptProbeOnlyPrologue(VMFrame &f)
 {
+    AutoAssertNoGC nogc;
     Probes::enterScript(f.cx, f.script(), f.script()->function(), f.fp());
 }
 
 void JS_FASTCALL
 stubs::ScriptProbeOnlyEpilogue(VMFrame &f)
 {
+    AutoAssertNoGC nogc;
     Probes::exitScript(f.cx, f.script(), f.script()->function(), f.fp());
 }
 
 void JS_FASTCALL
 stubs::CrossChunkShim(VMFrame &f, void *edge_)
 {
+    AssertCanGC();
     DebugOnly<CrossChunkEdge*> edge = (CrossChunkEdge *) edge_;
 
     mjit::ExpandInlineFrames(f.cx->compartment);
 
-    JSScript *script = f.script();
+    RootedScript script(f.cx, f.script());
     JS_ASSERT(edge->target < script->length);
     JS_ASSERT(script->code + edge->target == f.pc());
 
-    CompileStatus status = CanMethodJIT(f.cx, script, f.pc(), f.fp()->isConstructing(),
+    CompileStatus status = CanMethodJIT(f.cx, script, f.pc(),
+                                        f.fp()->isConstructing(),
                                         CompileRequest_Interpreter, f.fp());
     if (status == Compile_Error)
         THROW();
@@ -679,39 +737,6 @@ static const char *OpcodeNames[] = {
 };
 #endif
 
-static void
-FinishVarIncOp(VMFrame &f, RejoinState rejoin, Value ov, Value nv, Value *vp)
-{
-    /* Finish an increment operation on a LOCAL or ARG. These do not involve property accesses. */
-    JS_ASSERT(rejoin == REJOIN_POS || rejoin == REJOIN_BINARY);
-
-    JSContext *cx = f.cx;
-
-    JSOp op = JSOp(*f.pc());
-    JS_ASSERT(op == JSOP_LOCALINC || op == JSOP_INCLOCAL ||
-              op == JSOP_LOCALDEC || op == JSOP_DECLOCAL ||
-              op == JSOP_ARGINC || op == JSOP_INCARG ||
-              op == JSOP_ARGDEC || op == JSOP_DECARG);
-    const JSCodeSpec *cs = &js_CodeSpec[op];
-
-    if (rejoin == REJOIN_POS) {
-        double d = ov.toNumber();
-        double N = (cs->format & JOF_INC) ? 1 : -1;
-        if (!nv.setNumber(d + N))
-            types::TypeScript::MonitorOverflow(cx, f.script(), f.pc());
-    }
-
-    unsigned i = GET_SLOTNO(f.pc());
-    if (JOF_TYPE(cs->format) == JOF_LOCAL)
-        f.fp()->unaliasedLocal(i) = nv;
-    else if (f.fp()->script()->argsObjAliasesFormals())
-        f.fp()->argsObj().setArg(i, nv);
-    else
-        f.fp()->unaliasedFormal(i) = nv;
-
-    *vp = (cs->format & JOF_POST) ? ov : nv;
-}
-
 extern "C" void *
 js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VMFrame &f)
 {
@@ -729,14 +754,13 @@ js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VM
 
     JSContext *cx = f.cx;
     StackFrame *fp = f.regs.fp();
-    JSScript *script = fp->script();
+    RootedScript script(cx, fp->script());
 
     jsbytecode *pc = f.regs.pc;
 
     JSOp op = JSOp(*pc);
-    const JSCodeSpec *cs = &js_CodeSpec[op];
 
-    if (!script->ensureRanAnalysis(cx)) {
+    if (!JSScript::ensureRanAnalysis(cx, script)) {
         js_ReportOutOfMemory(cx);
         return js_InternalThrow(f);
     }
@@ -767,29 +791,6 @@ js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VM
     uint32_t nextDepth = UINT32_MAX;
     bool skipTrap = false;
 
-    if ((cs->format & (JOF_INC | JOF_DEC)) &&
-        (rejoin == REJOIN_POS || rejoin == REJOIN_BINARY)) {
-        /*
-         * We may reenter the interpreter while finishing the INC/DEC operation
-         * on a local or arg (property INC/DEC operations will rejoin into the
-         * decomposed version of the op.
-         */
-        JS_ASSERT(cs->format & (JOF_LOCAL | JOF_QARG));
-
-        nextDepth = analysis->getCode(nextpc).stackDepth;
-        enter.leave();
-
-        if (rejoin != REJOIN_BINARY || !analysis->incrementInitialValueObserved(pc)) {
-            /* Stack layout is 'V', 'N' or 'N+1' (only if the N is not needed) */
-            FinishVarIncOp(f, rejoin, nextsp[-1], nextsp[-1], &nextsp[-1]);
-        } else {
-            /* Stack layout is 'N N+1' */
-            FinishVarIncOp(f, rejoin, nextsp[-1], nextsp[0], &nextsp[-1]);
-        }
-
-        rejoin = REJOIN_FALLTHROUGH;
-    }
-
     switch (rejoin) {
       case REJOIN_SCRIPTED: {
         jsval_layout rval;
@@ -867,7 +868,10 @@ js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VM
         if (!obj)
             return js_InternalThrow(f);
         fp->thisValue() = ObjectValue(*obj);
+        /* FALLTHROUGH */
+      }
 
+      case REJOIN_THIS_CREATED: {
         Probes::enterScript(f.cx, f.script(), f.script()->function(), fp);
 
         if (script->debugMode) {
@@ -875,9 +879,13 @@ js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VM
             switch (status) {
               case JSTRAP_CONTINUE:
                 break;
-              case JSTRAP_RETURN:
-                *f.returnAddressLocation() = f.cx->jaegerRuntime().forceReturnFromExternC();
-                return NULL;
+              case JSTRAP_RETURN: {
+                /* Advance to the JSOP_STOP at the end of the script. */
+                f.regs.pc = script->code + script->length - 1;
+                nextDepth = 0;
+                JS_ASSERT(*f.regs.pc == JSOP_STOP);
+                break;
+              }
               case JSTRAP_THROW:
               case JSTRAP_ERROR:
                 return js_InternalThrow(f);
@@ -1005,10 +1013,15 @@ js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VM
              * portion of fun_hasInstance.
              */
             if (f.regs.sp[0].isPrimitive()) {
-                js_ReportValueError(cx, JSMSG_BAD_PROTOTYPE, -1, f.regs.sp[-1], NULL);
+                RootedValue val(cx, f.regs.sp[-1]);
+                js_ReportValueError(cx, JSMSG_BAD_PROTOTYPE, -1, val, NullPtr());
                 return js_InternalThrow(f);
             }
-            nextsp[-1].setBoolean(js_IsDelegate(cx, &f.regs.sp[0].toObject(), f.regs.sp[-2]));
+            bool isDelegate;
+            RootedObject obj(cx, &f.regs.sp[0].toObject());
+            if (!IsDelegate(cx, obj, f.regs.sp[-2], &isDelegate))
+                return js_InternalThrow(f);
+            nextsp[-1].setBoolean(isDelegate);
             f.regs.pc = nextpc;
             break;
           }
@@ -1019,18 +1032,6 @@ js_InternalInterpret(void *returnData, void *returnType, void *returnReg, js::VM
         }
         break;
 
-      case REJOIN_POS:
-        /* Convert-to-number which might be part of an INC* op. */
-        JS_ASSERT(op == JSOP_POS);
-        f.regs.pc = nextpc;
-        break;
-
-      case REJOIN_BINARY:
-        /* Binary arithmetic op which might be part of an INC* op. */
-        JS_ASSERT(op == JSOP_ADD || op == JSOP_SUB || op == JSOP_MUL || op == JSOP_DIV);
-        f.regs.pc = nextpc;
-        break;
-
       case REJOIN_BRANCH: {
         /*
          * This must be an opcode fused with IFNE/IFEQ. Unfused IFNE/IFEQ are
