@@ -9,6 +9,7 @@
 #define GlobalObject_h___
 
 #include "mozilla/Attributes.h"
+#include "mozilla/DebugOnly.h"
 
 #include "jsarray.h"
 #include "jsbool.h"
@@ -95,7 +96,10 @@ class GlobalObject : public JSObject
     static const unsigned GENERATOR_PROTO         = ELEMENT_ITERATOR_PROTO + 1;
     static const unsigned MAP_ITERATOR_PROTO      = GENERATOR_PROTO + 1;
     static const unsigned SET_ITERATOR_PROTO      = MAP_ITERATOR_PROTO + 1;
-    static const unsigned REGEXP_STATICS          = SET_ITERATOR_PROTO + 1;
+    static const unsigned COLLATOR_PROTO          = SET_ITERATOR_PROTO + 1;
+    static const unsigned NUMBER_FORMAT_PROTO     = COLLATOR_PROTO + 1;
+    static const unsigned DATE_TIME_FORMAT_PROTO  = NUMBER_FORMAT_PROTO + 1;
+    static const unsigned REGEXP_STATICS          = DATE_TIME_FORMAT_PROTO + 1;
     static const unsigned FUNCTION_NS             = REGEXP_STATICS + 1;
     static const unsigned RUNTIME_CODEGEN_ENABLED = FUNCTION_NS + 1;
     static const unsigned DEBUGGERS               = RUNTIME_CODEGEN_ENABLED + 1;
@@ -310,13 +314,24 @@ class GlobalObject : public JSObject
         return &self->getPrototype(key).toObject();
     }
 
-    JSObject *getIteratorPrototype() {
-        return &getPrototype(JSProto_Iterator).toObject();
+    JSObject *getOrCreateIntlObject(JSContext *cx) {
+        return getOrCreateObject(cx, JSProto_Intl, initIntlObject);
     }
 
-    JSObject *getIntrinsicsHolder() {
-        JS_ASSERT(!getSlotRef(INTRINSICS).isUndefined());
-        return &getSlotRef(INTRINSICS).toObject();
+    JSObject *getOrCreateCollatorPrototype(JSContext *cx) {
+        return getOrCreateObject(cx, COLLATOR_PROTO, initCollatorProto);
+    }
+
+    JSObject *getOrCreateNumberFormatPrototype(JSContext *cx) {
+        return getOrCreateObject(cx, NUMBER_FORMAT_PROTO, initNumberFormatProto);
+    }
+
+    JSObject *getOrCreateDateTimeFormatPrototype(JSContext *cx) {
+        return getOrCreateObject(cx, DATE_TIME_FORMAT_PROTO, initDateTimeFormatProto);
+    }
+
+    JSObject *getIteratorPrototype() {
+        return &getPrototype(JSProto_Iterator).toObject();
     }
 
   private:
@@ -362,26 +377,34 @@ class GlobalObject : public JSObject
         return &self->getPrototype(JSProto_DataView).toObject();
     }
 
-    bool hasIntrinsicFunction(JSContext *cx, PropertyName *name) {
-        RootedObject holder(cx, &getSlotRef(INTRINSICS).toObject());
-        Value fun = NullValue();
-        return HasDataProperty(cx, holder, NameToId(name), &fun);
+    JSObject *intrinsicsHolder() {
+        JS_ASSERT(!getSlotRef(INTRINSICS).isUndefined());
+        return &getSlotRef(INTRINSICS).toObject();
     }
 
     bool getIntrinsicValue(JSContext *cx, PropertyName *name, MutableHandleValue value) {
-        RootedObject holder(cx, &getSlotRef(INTRINSICS).toObject());
+        RootedObject holder(cx, intrinsicsHolder());
         RootedId id(cx, NameToId(name));
         if (HasDataProperty(cx, holder, id, value.address()))
             return true;
-        bool ok = cx->runtime->cloneSelfHostedValueById(cx, id, holder, value);
-        if (!ok)
+        Rooted<PropertyName*> rootedName(cx, name);
+        if (!cx->runtime->cloneSelfHostedValue(cx, rootedName, value))
             return false;
-
-        ok = JS_DefinePropertyById(cx, holder, id, value, NULL, NULL, 0);
+        mozilla::DebugOnly<bool> ok = JS_DefinePropertyById(cx, holder, id, value, NULL, NULL, 0);
         JS_ASSERT(ok);
         return true;
     }
 
+    bool setIntrinsicValue(JSContext *cx, PropertyName *name, HandleValue value) {
+#ifdef DEBUG
+        RootedObject self(cx, this);
+        JS_ASSERT(cx->runtime->isSelfHostingGlobal(self));
+#endif
+        RootedObject holder(cx, intrinsicsHolder());
+        RootedValue valCopy(cx, value);
+        return JSObject::setProperty(cx, holder, holder, name, &valCopy, false);
+     }
+
     inline RegExpStatics *getRegExpStatics() const;
 
     JSObject *getThrowTypeError() const {
@@ -423,6 +446,12 @@ class GlobalObject : public JSObject
     static bool initMapIteratorProto(JSContext *cx, Handle<GlobalObject*> global);
     static bool initSetIteratorProto(JSContext *cx, Handle<GlobalObject*> global);
 
+    // Implemented in Intl.cpp.
+    static bool initIntlObject(JSContext *cx, Handle<GlobalObject*> global);
+    static bool initCollatorProto(JSContext *cx, Handle<GlobalObject*> global);
+    static bool initNumberFormatProto(JSContext *cx, Handle<GlobalObject*> global);
+    static bool initDateTimeFormatProto(JSContext *cx, Handle<GlobalObject*> global);
+
     static bool initStandardClasses(JSContext *cx, Handle<GlobalObject*> global);
 
     typedef js::Vector<js::Debugger *, 0, js::SystemAllocPolicy> DebuggerVector;
