@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2011 The WebRTC project authors. All Rights Reserved.
+ *  Copyright (c) 2012 The WebRTC project authors. All Rights Reserved.
  *
  *  Use of this source code is governed by a BSD-style license
  *  that can be found in the LICENSE file in the root of the source
@@ -398,10 +398,14 @@ GenericCodecTest::Perform(CmdArgs& args)
     /********************************/
     /* Encoder Packet Size Test     */
     /********************************/
-    RtpRtcp& rtpModule = *RtpRtcp::CreateRtpRtcp(1, false);
-    TEST(rtpModule.InitSender() == 0);
     RTPSendCallback_SizeTest sendCallback;
-    rtpModule.RegisterSendTransport(&sendCallback);
+
+    RtpRtcp::Configuration configuration;
+    configuration.id = 1;
+    configuration.audio = false;
+    configuration.outgoing_transport = &sendCallback;
+
+    RtpRtcp& rtpModule = *RtpRtcp::CreateRtpRtcp(configuration);
 
     VCMRTPEncodeCompleteCallback encCompleteCallback(&rtpModule);
     _vcm->InitializeSender();
@@ -485,7 +489,7 @@ GenericCodecTest::Perform(CmdArgs& args)
         IncrementDebugClock(_frameRate);
     } // first frame encoded
 
-    RtpRtcp::DestroyRtpRtcp(&rtpModule);
+    delete &rtpModule;
     Print();
     delete tmpBuffer;
     delete _decodeCallback;
@@ -566,6 +570,7 @@ VCMEncComplete_KeyReqTest::SendData(
         const FrameType frameType,
         const WebRtc_UWord8 payloadType,
         const WebRtc_UWord32 timeStamp,
+        int64_t capture_time_ms,
         const WebRtc_UWord8* payloadData,
         const WebRtc_UWord32 payloadSize,
         const RTPFragmentationHeader& /*fragmentationHeader*/,
