@@ -4,13 +4,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "nsCache.h"
 #include "nsMemoryCacheDevice.h"
 #include "nsCacheService.h"
 #include "nsICacheService.h"
 #include "nsIStorageStream.h"
 #include "nsICacheVisitor.h"
 #include "nsCRT.h"
-#include "nsCache.h"
 #include "nsReadableUtils.h"
 #include "mozilla/Telemetry.h"
 
@@ -77,7 +77,7 @@ nsMemoryCacheDevice::Shutdown()
             PR_REMOVE_AND_INIT_LINK(entry);
         
             // update statistics
-            PRInt32 memoryRecovered = (PRInt32)entry->DataSize();
+            int32_t memoryRecovered = (int32_t)entry->DataSize();
             mTotalSize    -= memoryRecovered;
             mInactiveSize -= memoryRecovered;
             --mEntryCount;
@@ -110,9 +110,9 @@ nsMemoryCacheDevice::GetDeviceID()
 nsCacheEntry *
 nsMemoryCacheDevice::FindEntry(nsCString * key, bool *collision)
 {
-    mozilla::Telemetry::AutoTimer<mozilla::Telemetry::CACHE_MEMORY_SEARCH> timer;
+    mozilla::Telemetry::AutoTimer<mozilla::Telemetry::CACHE_MEMORY_SEARCH_2> timer;
     nsCacheEntry * entry = mMemCacheEntries.GetEntry(key);
-    if (!entry)  return nsnull;
+    if (!entry)  return nullptr;
 
     // move entry to the tail of an eviction list
     PR_REMOVE_AND_INIT_LINK(entry);
@@ -198,7 +198,7 @@ nsMemoryCacheDevice::DoomEntry(nsCacheEntry * entry)
 nsresult
 nsMemoryCacheDevice::OpenInputStreamForEntry( nsCacheEntry *    entry,
                                               nsCacheAccessMode mode,
-                                              PRUint32          offset,
+                                              uint32_t          offset,
                                               nsIInputStream ** result)
 {
     NS_ENSURE_ARG_POINTER(entry);
@@ -214,7 +214,7 @@ nsMemoryCacheDevice::OpenInputStreamForEntry( nsCacheEntry *    entry,
             return rv;
     }
     else {
-        rv = NS_NewStorageStream(4096, PRUint32(-1), getter_AddRefs(storage));
+        rv = NS_NewStorageStream(4096, uint32_t(-1), getter_AddRefs(storage));
         if (NS_FAILED(rv))
             return rv;
         entry->SetData(storage);
@@ -227,7 +227,7 @@ nsMemoryCacheDevice::OpenInputStreamForEntry( nsCacheEntry *    entry,
 nsresult
 nsMemoryCacheDevice::OpenOutputStreamForEntry( nsCacheEntry *     entry,
                                                nsCacheAccessMode  mode,
-                                               PRUint32           offset,
+                                               uint32_t           offset,
                                                nsIOutputStream ** result)
 {
     NS_ENSURE_ARG_POINTER(entry);
@@ -243,7 +243,7 @@ nsMemoryCacheDevice::OpenOutputStreamForEntry( nsCacheEntry *     entry,
             return rv;
     }
     else {
-        rv = NS_NewStorageStream(4096, PRUint32(-1), getter_AddRefs(storage));
+        rv = NS_NewStorageStream(4096, uint32_t(-1), getter_AddRefs(storage));
         if (NS_FAILED(rv))
             return rv;
         entry->SetData(storage);
@@ -261,7 +261,7 @@ nsMemoryCacheDevice::GetFileForEntry( nsCacheEntry *    entry,
 }
 
 bool
-nsMemoryCacheDevice::EntryIsTooBig(PRInt64 entrySize)
+nsMemoryCacheDevice::EntryIsTooBig(int64_t entrySize)
 {
     CACHE_LOG_DEBUG(("nsMemoryCacheDevice::EntryIsTooBig "
                      "[size=%d max=%d soft=%d]\n",
@@ -279,11 +279,11 @@ nsMemoryCacheDevice::TotalSize()
 }
 
 nsresult
-nsMemoryCacheDevice::OnDataSizeChange( nsCacheEntry * entry, PRInt32 deltaSize)
+nsMemoryCacheDevice::OnDataSizeChange( nsCacheEntry * entry, int32_t deltaSize)
 {
     if (entry->IsStreamData()) {
         // we have the right to refuse or pre-evict
-        PRUint32  newSize = entry->DataSize() + deltaSize;
+        uint32_t  newSize = entry->DataSize() + deltaSize;
         if (EntryIsTooBig(newSize)) {
 #ifdef DEBUG
             nsresult rv =
@@ -309,7 +309,7 @@ nsMemoryCacheDevice::OnDataSizeChange( nsCacheEntry * entry, PRInt32 deltaSize)
 
 
 void
-nsMemoryCacheDevice::AdjustMemoryLimits(PRInt32  softLimit, PRInt32  hardLimit)
+nsMemoryCacheDevice::AdjustMemoryLimits(int32_t  softLimit, int32_t  hardLimit)
 {
     mSoftLimit = softLimit;
     mHardLimit = hardLimit;
@@ -331,7 +331,7 @@ nsMemoryCacheDevice::EvictEntry(nsCacheEntry * entry, bool deleteEntry)
     PR_REMOVE_AND_INIT_LINK(entry);
     
     // update statistics
-    PRInt32 memoryRecovered = (PRInt32)entry->DataSize();
+    int32_t memoryRecovered = (int32_t)entry->DataSize();
     mTotalSize    -= memoryRecovered;
     if (!entry->IsDoomed())
         mInactiveSize -= memoryRecovered;
@@ -353,9 +353,9 @@ nsMemoryCacheDevice::EvictEntriesIfNecessary(void)
     if ((mTotalSize < mHardLimit) && (mInactiveSize < mSoftLimit))
         return;
 
-    PRUint32 now = SecondsFromPRTime(PR_Now());
-    PRUint64 entryCost = 0;
-    PRUint64 maxCost = 0;
+    uint32_t now = SecondsFromPRTime(PR_Now());
+    uint64_t entryCost = 0;
+    uint64_t maxCost = 0;
     do {
         // LRU-SP eviction selection: Check the head of each segment (each
         // eviction list, kept in LRU order) and select the maximal-cost
@@ -371,9 +371,9 @@ nsMemoryCacheDevice::EvictEntriesIfNecessary(void)
             }
 
             if (entry != &mEvictionList[i]) {
-                entryCost = (PRUint64)
+                entryCost = (uint64_t)
                     (now - entry->LastFetched()) * entry->DataSize() / 
-                    PR_MAX(1, entry->FetchCount());
+                    NS_MAX(1, entry->FetchCount());
                 if (!maxEntry || (entryCost > maxCost)) {
                     maxEntry = entry;
                     maxCost = entryCost;
@@ -391,7 +391,7 @@ nsMemoryCacheDevice::EvictEntriesIfNecessary(void)
 
 
 int
-nsMemoryCacheDevice::EvictionList(nsCacheEntry * entry, PRInt32  deltaSize)
+nsMemoryCacheDevice::EvictionList(nsCacheEntry * entry, int32_t  deltaSize)
 {
     // favor items which never expire by putting them in the lowest-index queue
     if (entry->ExpirationTime() == nsICache::NO_EXPIRATION_TIME)
@@ -399,8 +399,8 @@ nsMemoryCacheDevice::EvictionList(nsCacheEntry * entry, PRInt32  deltaSize)
 
     // compute which eviction queue this entry should go into,
     // based on floor(log2(size/nref))
-    PRInt32  size       = deltaSize + (PRInt32)entry->DataSize();
-    PRInt32  fetchCount = NS_MAX(1, entry->FetchCount());
+    int32_t  size       = deltaSize + (int32_t)entry->DataSize();
+    int32_t  fetchCount = NS_MAX(1, entry->FetchCount());
 
     return NS_MIN(PR_FloorLog2(size / fetchCount), kQueueCount - 1);
 }
@@ -442,26 +442,44 @@ nsMemoryCacheDevice::Visit(nsICacheVisitor * visitor)
 }
 
 
+static bool
+IsEntryPrivate(nsCacheEntry* entry, void* args)
+{
+    return entry->IsPrivate();
+}
+
+struct ClientIDArgs {
+    const char* clientID;
+    uint32_t prefixLength;
+};
+
+static bool
+EntryMatchesClientID(nsCacheEntry* entry, void* args)
+{
+    const char * clientID = static_cast<ClientIDArgs*>(args)->clientID;
+    uint32_t prefixLength = static_cast<ClientIDArgs*>(args)->prefixLength;
+    const char * key = entry->Key()->get();
+    return !clientID || nsCRT::strncmp(clientID, key, prefixLength) == 0;
+}
+
 nsresult
-nsMemoryCacheDevice::EvictEntries(const char * clientID)
+nsMemoryCacheDevice::DoEvictEntries(bool (*matchFn)(nsCacheEntry* entry, void* args), void* args)
 {
     nsCacheEntry * entry;
-    PRUint32 prefixLength = (clientID ? strlen(clientID) : 0);
 
     for (int i = kQueueCount - 1; i >= 0; --i) {
         PRCList * elem = PR_LIST_HEAD(&mEvictionList[i]);
         while (elem != &mEvictionList[i]) {
             entry = (nsCacheEntry *)elem;
             elem = PR_NEXT_LINK(elem);
-            
-            const char * key = entry->Key()->get();
-            if (clientID && nsCRT::strncmp(clientID, key, prefixLength) != 0)
+
+            if (!matchFn(entry, args))
                 continue;
             
             if (entry->IsInUse()) {
                 nsresult rv = nsCacheService::DoomEntry(entry);
                 if (NS_FAILED(rv)) {
-                    CACHE_LOG_WARNING(("memCache->EvictEntries() aborted: rv =%x", rv));
+                    CACHE_LOG_WARNING(("memCache->DoEvictEntries() aborted: rv =%x", rv));
                     return rv;
                 }
             } else {
@@ -473,18 +491,31 @@ nsMemoryCacheDevice::EvictEntries(const char * clientID)
     return NS_OK;
 }
 
+nsresult
+nsMemoryCacheDevice::EvictEntries(const char * clientID)
+{
+    ClientIDArgs args = {clientID, clientID ? uint32_t(strlen(clientID)) : 0};
+    return DoEvictEntries(&EntryMatchesClientID, &args);
+}
+
+nsresult
+nsMemoryCacheDevice::EvictPrivateEntries()
+{
+    return DoEvictEntries(&IsEntryPrivate, NULL);
+}
+
 
 // WARNING: SetCapacity can get called before Init()
 void
-nsMemoryCacheDevice::SetCapacity(PRInt32  capacity)
+nsMemoryCacheDevice::SetCapacity(int32_t  capacity)
 {
-    PRInt32 hardLimit = capacity * 1024;  // convert k into bytes
-    PRInt32 softLimit = (hardLimit * 9) / 10;
+    int32_t hardLimit = capacity * 1024;  // convert k into bytes
+    int32_t softLimit = (hardLimit * 9) / 10;
     AdjustMemoryLimits(softLimit, hardLimit);
 }
 
 void
-nsMemoryCacheDevice::SetMaxEntrySize(PRInt32 maxSizeInKilobytes)
+nsMemoryCacheDevice::SetMaxEntrySize(int32_t maxSizeInKilobytes)
 {
     // Internal unit is bytes. Changing this only takes effect *after* the
     // change and has no consequences for existing cache-entries
@@ -496,9 +527,9 @@ nsMemoryCacheDevice::SetMaxEntrySize(PRInt32 maxSizeInKilobytes)
 
 #ifdef DEBUG
 static PLDHashOperator
-CountEntry(PLDHashTable * table, PLDHashEntryHdr * hdr, PRUint32 number, void * arg)
+CountEntry(PLDHashTable * table, PLDHashEntryHdr * hdr, uint32_t number, void * arg)
 {
-    PRInt32 *entryCount = (PRInt32 *)arg;
+    int32_t *entryCount = (int32_t *)arg;
     ++(*entryCount);
     return PL_DHASH_NEXT;
 }
@@ -508,7 +539,7 @@ nsMemoryCacheDevice::CheckEntryCount()
 {
     if (!mInitialized)  return;
 
-    PRInt32 evictionListCount = 0;
+    int32_t evictionListCount = 0;
     for (int i=0; i<kQueueCount; ++i) {
         PRCList * elem = PR_LIST_HEAD(&mEvictionList[i]);
         while (elem != &mEvictionList[i]) {
@@ -518,7 +549,7 @@ nsMemoryCacheDevice::CheckEntryCount()
     }
     NS_ASSERTION(mEntryCount == evictionListCount, "### mem cache badness");
 
-    PRInt32 entryCount = 0;
+    int32_t entryCount = 0;
     mMemCacheEntries.VisitEntries(CountEntry, &entryCount);
     NS_ASSERTION(mEntryCount == entryCount, "### mem cache badness");    
 }
@@ -562,28 +593,28 @@ nsMemoryCacheDeviceInfo::GetUsageReport(char ** result)
 
 
 NS_IMETHODIMP
-nsMemoryCacheDeviceInfo::GetEntryCount(PRUint32 * result)
+nsMemoryCacheDeviceInfo::GetEntryCount(uint32_t * result)
 {
     NS_ENSURE_ARG_POINTER(result);
     // XXX compare calculated count vs. mEntryCount
-    *result = (PRUint32)mDevice->mEntryCount;
+    *result = (uint32_t)mDevice->mEntryCount;
     return NS_OK;
 }
 
 
 NS_IMETHODIMP
-nsMemoryCacheDeviceInfo::GetTotalSize(PRUint32 * result)
+nsMemoryCacheDeviceInfo::GetTotalSize(uint32_t * result)
 {
     NS_ENSURE_ARG_POINTER(result);
-    *result = (PRUint32)mDevice->mTotalSize;
+    *result = (uint32_t)mDevice->mTotalSize;
     return NS_OK;
 }
 
 
 NS_IMETHODIMP
-nsMemoryCacheDeviceInfo::GetMaximumSize(PRUint32 * result)
+nsMemoryCacheDeviceInfo::GetMaximumSize(uint32_t * result)
 {
     NS_ENSURE_ARG_POINTER(result);
-    *result = (PRUint32)mDevice->mHardLimit;
+    *result = (uint32_t)mDevice->mHardLimit;
     return NS_OK;
 }
