@@ -15,7 +15,7 @@
 #include "nsString.h"
 #include "nsAutoPtr.h"
 #include "nsNetCID.h"
-#include "nsNetError.h"
+#include "nsError.h"
 #include "nsDNSPrefetch.h"
 #include "nsThreadUtils.h"
 #include "nsIProtocolProxyService.h"
@@ -27,10 +27,10 @@
 #include "nsIOService.h"
 #include "nsCharSeparatedTokenizer.h"
 
-#include "mozilla/FunctionTimer.h"
 #include "mozilla/Attributes.h"
 
 using namespace mozilla;
+using namespace mozilla::net;
 
 static const char kPrefDnsCacheEntries[]    = "network.dnsCacheEntries";
 static const char kPrefDnsCacheExpiration[] = "network.dnsCacheExpiration";
@@ -51,8 +51,7 @@ public:
 
     nsDNSRecord(nsHostRecord *hostRecord)
         : mHostRecord(hostRecord)
-        , mIter(nsnull)
-        , mLastIter(nsnull)
+        , mIter(nullptr)
         , mIterGenCnt(-1)
         , mDone(false) {}
 
@@ -60,9 +59,7 @@ private:
     virtual ~nsDNSRecord() {}
 
     nsRefPtr<nsHostRecord>  mHostRecord;
-    void                   *mIter;       // enum ptr for PR_EnumerateAddrInfo
-    void                   *mLastIter;   // previous enum ptr, for use in
-                                         // getting addrinfo in ReportUnusable
+    NetAddrElement         *mIter;
     int                     mIterGenCnt; // the generation count of
                                          // mHostRecord->addr_info when we
                                          // start iterating
@@ -84,7 +81,7 @@ nsDNSRecord::GetCanonicalName(nsACString &result)
     {
         MutexAutoLock lock(mHostRecord->addr_info_lock);
         if (mHostRecord->addr_info)
-            cname = PR_GetCanonNameFromAddrInfo(mHostRecord->addr_info);
+            cname = mHostRecord->addr_info->mHostName;
         else
             cname = mHostRecord->host;
         result.Assign(cname);
@@ -93,46 +90,45 @@ nsDNSRecord::GetCanonicalName(nsACString &result)
 }
 
 NS_IMETHODIMP
-nsDNSRecord::GetNextAddr(PRUint16 port, PRNetAddr *addr)
+nsDNSRecord::GetNextAddr(uint16_t port, NetAddr *addr)
 {
-    // not a programming error to poke the DNS record when it has no more
-    // entries.  just fail without any debug warnings.  this enables consumers
-    // to enumerate the DNS record without calling HasMore.
-    if (mDone)
+    if (mDone) {
         return NS_ERROR_NOT_AVAILABLE;
+    }
 
     mHostRecord->addr_info_lock.Lock();
-    bool startedFresh = !mIter;
-
     if (mHostRecord->addr_info) {
-        if (!mIter)
+        if (mIterGenCnt != mHostRecord->addr_info_gencnt) {
+            // mHostRecord->addr_info has changed, restart the iteration.
+            mIter = nullptr;
             mIterGenCnt = mHostRecord->addr_info_gencnt;
-        else if (mIterGenCnt != mHostRecord->addr_info_gencnt) {
-            // mHostRecord->addr_info has changed, so mIter is invalid.
-            // Restart the iteration.  Alternatively, we could just fail.
-            mIter = nsnull;
-            mIterGenCnt = mHostRecord->addr_info_gencnt;
-            startedFresh = true;
         }
 
+        bool startedFresh = !mIter;
+
         do {
-            mLastIter = mIter;
-            mIter = PR_EnumerateAddrInfo(mIter, mHostRecord->addr_info,
-                                         port, addr);
+            if (!mIter) {
+                mIter = mHostRecord->addr_info->mAddresses.getFirst();
+            } else {
+                mIter = mIter->getNext();
+            }
         }
-        while (mIter && mHostRecord->Blacklisted(addr));
+        while (mIter && mHostRecord->Blacklisted(&mIter->mAddress));
 
-        if (startedFresh && !mIter) {
-            // if everything was blacklisted we want to reset the blacklist (and
+        if (!mIter && startedFresh) {
+            // If everything was blacklisted we want to reset the blacklist (and
             // likely relearn it) and return the first address. That is better
-            // than nothing
+            // than nothing.
             mHostRecord->ResetBlacklist();
-            mLastIter = nsnull;
-            mIter = PR_EnumerateAddrInfo(nsnull, mHostRecord->addr_info,
-                                         port, addr);
+            mIter = mHostRecord->addr_info->mAddresses.getFirst();
         }
-            
+
+        if (mIter) {
+            memcpy(addr, &mIter->mAddress, sizeof(NetAddr));
+        }
+
         mHostRecord->addr_info_lock.Unlock();
+
         if (!mIter) {
             mDone = true;
             return NS_ERROR_NOT_AVAILABLE;
@@ -140,76 +136,79 @@ nsDNSRecord::GetNextAddr(PRUint16 port, PRNetAddr *addr)
     }
     else {
         mHostRecord->addr_info_lock.Unlock();
+
         if (!mHostRecord->addr) {
             // Both mHostRecord->addr_info and mHostRecord->addr are null.
             // This can happen if mHostRecord->addr_info expired and the
             // attempt to reresolve it failed.
             return NS_ERROR_NOT_AVAILABLE;
         }
-        memcpy(addr, mHostRecord->addr, sizeof(PRNetAddr));
-        // set given port
-        port = PR_htons(port);
-        if (addr->raw.family == PR_AF_INET)
-            addr->inet.port = port;
-        else
-            addr->ipv6.port = port;
-        mDone = true; // no iterations
+        memcpy(addr, mHostRecord->addr, sizeof(NetAddr));
+        mDone = true;
     }
-        
-    return NS_OK; 
+
+    // set given port
+    port = htons(port);
+    if (addr->raw.family == AF_INET) {
+        addr->inet.port = port;
+    }
+    else if (addr->raw.family == AF_INET6) {
+        addr->inet6.port = port;
+    }
+
+    return NS_OK;
 }
 
 NS_IMETHODIMP
 nsDNSRecord::GetNextAddrAsString(nsACString &result)
 {
-    PRNetAddr addr;
+    NetAddr addr;
     nsresult rv = GetNextAddr(0, &addr);
     if (NS_FAILED(rv)) return rv;
 
-    char buf[64];
-    if (PR_NetAddrToString(&addr, buf, sizeof(buf)) == PR_SUCCESS) {
+    char buf[kIPv6CStrBufSize];
+    if (NetAddrToString(&addr, buf, sizeof(buf))) {
         result.Assign(buf);
         return NS_OK;
     }
-    NS_ERROR("PR_NetAddrToString failed unexpectedly");
+    NS_ERROR("NetAddrToString failed unexpectedly");
     return NS_ERROR_FAILURE; // conversion failed for some reason
 }
 
 NS_IMETHODIMP
 nsDNSRecord::HasMore(bool *result)
 {
-    if (mDone)
+    if (mDone) {
         *result = false;
-    else {
-        // unfortunately, NSPR does not provide a way for us to determine if
-        // there is another address other than to simply get the next address.
-        void *iterCopy = mIter;
-        void *iterLastCopy = mLastIter;
-        PRNetAddr addr;
-        *result = NS_SUCCEEDED(GetNextAddr(0, &addr));
-        mIter = iterCopy; // backup iterator
-        mLastIter = iterLastCopy; // backup iterator
-        mDone = false;
+        return NS_OK;
     }
+
+    NetAddrElement *iterCopy = mIter;
+
+    NetAddr addr;
+    *result = NS_SUCCEEDED(GetNextAddr(0, &addr));
+
+    mIter = iterCopy;
+    mDone = false;
+
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsDNSRecord::Rewind()
 {
-    mIter = nsnull;
-    mLastIter = nsnull;
+    mIter = nullptr;
     mIterGenCnt = -1;
     mDone = false;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsDNSRecord::ReportUnusable(PRUint16 aPort)
+nsDNSRecord::ReportUnusable(uint16_t aPort)
 {
     // right now we don't use the port in the blacklist
 
-    mHostRecord->addr_info_lock.Lock();
+    MutexAutoLock lock(mHostRecord->addr_info_lock);
 
     // Check that we are using a real addr_info (as opposed to a single
     // constant address), and that the generation count is valid. Otherwise,
@@ -217,14 +216,9 @@ nsDNSRecord::ReportUnusable(PRUint16 aPort)
 
     if (mHostRecord->addr_info &&
         mIterGenCnt == mHostRecord->addr_info_gencnt) {
-        PRNetAddr addr;
-        void *id = PR_EnumerateAddrInfo(mLastIter, mHostRecord->addr_info,
-                                        aPort, &addr);
-        if (id)
-            mHostRecord->ReportUnusable(&addr);
+        mHostRecord->ReportUnusable(&mIter->mAddress);
     }
-    
-    mHostRecord->addr_info_lock.Unlock();
+
     return NS_OK;
 }
 
@@ -240,8 +234,8 @@ public:
     nsDNSAsyncRequest(nsHostResolver   *res,
                       const nsACString &host,
                       nsIDNSListener   *listener,
-                      PRUint16          flags,
-                      PRUint16          af)
+                      uint16_t          flags,
+                      uint16_t          af)
         : mResolver(res)
         , mHost(host)
         , mListener(listener)
@@ -258,8 +252,8 @@ public:
     nsRefPtr<nsHostResolver> mResolver;
     nsCString                mHost; // hostname we're resolving
     nsCOMPtr<nsIDNSListener> mListener;
-    PRUint16                 mFlags;
-    PRUint16                 mAF;
+    uint16_t                 mFlags;
+    uint16_t                 mAF;
 };
 
 void
@@ -279,7 +273,7 @@ nsDNSAsyncRequest::OnLookupComplete(nsHostResolver *resolver,
     }
 
     mListener->OnLookupComplete(this, rec, status);
-    mListener = nsnull;
+    mListener = nullptr;
 
     // release the reference to ourselves that was added before we were
     // handed off to the host resolver.
@@ -350,6 +344,7 @@ nsDNSSyncRequest::EqualsAsyncListener(nsIDNSListener *aListener)
 nsDNSService::nsDNSService()
     : mLock("nsDNSServer.mLock")
     , mFirstTime(true)
+    , mOffline(false)
 {
 }
 
@@ -363,14 +358,14 @@ NS_IMPL_THREADSAFE_ISUPPORTS3(nsDNSService, nsIDNSService, nsPIDNSService,
 NS_IMETHODIMP
 nsDNSService::Init()
 {
-    NS_TIME_FUNCTION;
-
+    if (mResolver)
+        return NS_OK;
     NS_ENSURE_TRUE(!mResolver, NS_ERROR_ALREADY_INITIALIZED);
 
     // prefs
-    PRUint32 maxCacheEntries  = 400;
-    PRUint32 maxCacheLifetime = 2; // minutes
-    PRUint32 lifetimeGracePeriod = 1;
+    uint32_t maxCacheEntries  = 400;
+    uint32_t maxCacheLifetime = 2; // minutes
+    uint32_t lifetimeGracePeriod = 1;
     bool     enableIDN        = true;
     bool     disableIPv6      = false;
     bool     disablePrefetch  = false;
@@ -382,9 +377,9 @@ nsDNSService::Init()
     // read prefs
     nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
     if (prefs) {
-        PRInt32 val;
+        int32_t val;
         if (NS_SUCCEEDED(prefs->GetIntPref(kPrefDnsCacheEntries, &val)))
-            maxCacheEntries = (PRUint32) val;
+            maxCacheEntries = (uint32_t) val;
         if (NS_SUCCEEDED(prefs->GetIntPref(kPrefDnsCacheExpiration, &val)))
             maxCacheLifetime = val / 60; // convert from seconds to minutes
         if (NS_SUCCEEDED(prefs->GetIntPref(kPrefDnsCacheGrace, &val)))
@@ -475,13 +470,41 @@ nsDNSService::Shutdown()
     {
         MutexAutoLock lock(mLock);
         res = mResolver;
-        mResolver = nsnull;
+        mResolver = nullptr;
     }
     if (res)
         res->Shutdown();
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDNSService::GetOffline(bool *offline)
+{
+    *offline = mOffline;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDNSService::SetOffline(bool offline)
+{
+    mOffline = offline;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDNSService::GetPrefetchEnabled(bool *outVal)
+{
+    *outVal = !mDisablePrefetch;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDNSService::SetPrefetchEnabled(bool inVal)
+{
+    mDisablePrefetch = !inVal;
+    return NS_OK;
+}
+
 namespace {
 
 class DNSListenerProxy MOZ_FINAL : public nsIDNSListener
@@ -557,7 +580,7 @@ DNSListenerProxy::OnLookupCompleteRunnable::Run()
 
 NS_IMETHODIMP
 nsDNSService::AsyncResolve(const nsACString  &hostname,
-                           PRUint32           flags,
+                           uint32_t           flags,
                            nsIDNSListener    *listener,
                            nsIEventTarget    *target,
                            nsICancelable    **result)
@@ -580,6 +603,9 @@ nsDNSService::AsyncResolve(const nsACString  &hostname,
     if (!res)
         return NS_ERROR_OFFLINE;
 
+    if (mOffline)
+        flags |= RESOLVE_OFFLINE;
+
     const nsACString *hostPtr = &hostname;
 
     if (localDomain) {
@@ -587,7 +613,7 @@ nsDNSService::AsyncResolve(const nsACString  &hostname,
     }
 
     nsresult rv;
-    nsCAutoString hostACE;
+    nsAutoCString hostACE;
     if (idn && !IsASCII(*hostPtr)) {
         if (NS_SUCCEEDED(idn->ConvertUTF8toACE(*hostPtr, hostACE)))
             hostPtr = &hostACE;
@@ -597,7 +623,7 @@ nsDNSService::AsyncResolve(const nsACString  &hostname,
       listener = new DNSListenerProxy(listener, target);
     }
 
-    PRUint16 af = GetAFForLookup(*hostPtr, flags);
+    uint16_t af = GetAFForLookup(*hostPtr, flags);
 
     nsDNSAsyncRequest *req =
             new nsDNSAsyncRequest(res, *hostPtr, listener, flags, af);
@@ -617,7 +643,7 @@ nsDNSService::AsyncResolve(const nsACString  &hostname,
 
 NS_IMETHODIMP
 nsDNSService::CancelAsyncResolve(const nsACString  &aHostname,
-                                 PRUint32           aFlags,
+                                 uint32_t           aFlags,
                                  nsIDNSListener    *aListener,
                                  nsresult           aReason)
 {
@@ -639,13 +665,13 @@ nsDNSService::CancelAsyncResolve(const nsACString  &aHostname,
 
     nsCString hostname(aHostname);
 
-    nsCAutoString hostACE;
+    nsAutoCString hostACE;
     if (idn && !IsASCII(aHostname)) {
         if (NS_SUCCEEDED(idn->ConvertUTF8toACE(aHostname, hostACE)))
             hostname = hostACE;
     }
 
-    PRUint16 af = GetAFForLookup(hostname, aFlags);
+    uint16_t af = GetAFForLookup(hostname, aFlags);
 
     res->CancelAsyncRequest(hostname.get(), aFlags, af, aListener, aReason);
     return NS_OK;
@@ -653,7 +679,7 @@ nsDNSService::CancelAsyncResolve(const nsACString  &aHostname,
 
 NS_IMETHODIMP
 nsDNSService::Resolve(const nsACString &hostname,
-                      PRUint32          flags,
+                      uint32_t          flags,
                       nsIDNSRecord    **result)
 {
     // grab reference to global host resolver and IDN service.  beware
@@ -669,6 +695,9 @@ nsDNSService::Resolve(const nsACString &hostname,
     }
     NS_ENSURE_TRUE(res, NS_ERROR_OFFLINE);
 
+    if (mOffline)
+        flags |= RESOLVE_OFFLINE;
+
     const nsACString *hostPtr = &hostname;
 
     if (localDomain) {
@@ -676,7 +705,7 @@ nsDNSService::Resolve(const nsACString &hostname,
     }
 
     nsresult rv;
-    nsCAutoString hostACE;
+    nsAutoCString hostACE;
     if (idn && !IsASCII(*hostPtr)) {
         if (NS_SUCCEEDED(idn->ConvertUTF8toACE(*hostPtr, hostACE)))
             hostPtr = &hostACE;
@@ -697,7 +726,7 @@ nsDNSService::Resolve(const nsACString &hostname,
     PR_EnterMonitor(mon);
     nsDNSSyncRequest syncReq(mon);
 
-    PRUint16 af = GetAFForLookup(*hostPtr, flags);
+    uint16_t af = GetAFForLookup(*hostPtr, flags);
 
     rv = res->ResolveHost(PromiseFlatCString(*hostPtr).get(), flags, af, &syncReq);
     if (NS_SUCCEEDED(rv)) {
@@ -756,19 +785,19 @@ nsDNSService::Observe(nsISupports *subject, const char *topic, const PRUnichar *
     return NS_OK;
 }
 
-PRUint16
-nsDNSService::GetAFForLookup(const nsACString &host, PRUint32 flags)
+uint16_t
+nsDNSService::GetAFForLookup(const nsACString &host, uint32_t flags)
 {
     if (mDisableIPv6 || (flags & RESOLVE_DISABLE_IPV6))
         return PR_AF_INET;
 
     MutexAutoLock lock(mLock);
 
-    PRUint16 af = PR_AF_UNSPEC;
+    uint16_t af = PR_AF_UNSPEC;
 
     if (!mIPv4OnlyDomains.IsEmpty()) {
         const char *domain, *domainEnd, *end;
-        PRUint32 hostLen, domainLen;
+        uint32_t hostLen, domainLen;
 
         // see if host is in one of the IPv4-only domains
         domain = mIPv4OnlyDomains.BeginReading();
@@ -810,3 +839,11 @@ nsDNSService::GetAFForLookup(const nsACString &host, PRUint32 flags)
 
     return af;
 }
+
+NS_IMETHODIMP
+nsDNSService::GetDNSCacheEntries(nsTArray<mozilla::net::DNSCacheEntries> *args)
+{
+    NS_ENSURE_TRUE(mResolver, NS_ERROR_NOT_INITIALIZED);
+    mResolver->GetDNSCacheEntries(args);
+    return NS_OK;
+}
