@@ -30,6 +30,7 @@
 #include "nsThreadUtils.h"
 #include "nsXPCOMStrings.h"
 #include "nsProxyRelease.h"
+#include "mozilla/DebugOnly.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/Telemetry.h"
@@ -41,6 +42,7 @@
 #include "nsIPrincipal.h"
 #include "Classifier.h"
 #include "ProtocolParser.h"
+#include "nsContentUtils.h"
 
 using namespace mozilla;
 using namespace mozilla::safebrowsing;
@@ -116,7 +118,7 @@ public:
   NS_DECL_NSIURLCLASSIFIERDBSERVICE
   NS_DECL_NSIURLCLASSIFIERDBSERVICEWORKER
 
-  nsresult Init(int32_t gethashNoise, nsCOMPtr<nsIFile> aCacheDir,
+  nsresult Init(uint32_t aGethashNoise, nsCOMPtr<nsIFile> aCacheDir,
                 bool aPerClientRandomize);
 
   // Queue a lookup for the worker to perform, called in the main thread.
@@ -150,7 +152,7 @@ private:
 
   nsresult AddNoise(const Prefix aPrefix,
                     const nsCString tableName,
-                    int32_t aCount,
+                    uint32_t aCount,
                     LookupResultArray& results);
 
   nsCOMPtr<nsICryptoHash> mCryptoHash;
@@ -184,7 +186,7 @@ private:
   uint32_t mHashKey;
 
   // The number of noise entries to add to the set of lookup results.
-  int32_t mGethashNoise;
+  uint32_t mGethashNoise;
 
   // Randomize clients with a key or not.
   bool mPerClientRandomize;
@@ -224,11 +226,11 @@ nsUrlClassifierDBServiceWorker::~nsUrlClassifierDBServiceWorker()
 }
 
 nsresult
-nsUrlClassifierDBServiceWorker::Init(int32_t gethashNoise,
+nsUrlClassifierDBServiceWorker::Init(uint32_t aGethashNoise,
                                      nsCOMPtr<nsIFile> aCacheDir,
                                      bool aPerClientRandomize)
 {
-  mGethashNoise = gethashNoise;
+  mGethashNoise = aGethashNoise;
   mCacheDir = aCacheDir;
   mPerClientRandomize = aPerClientRandomize;
 
@@ -357,7 +359,7 @@ nsUrlClassifierDBServiceWorker::HandlePendingLookups()
 nsresult
 nsUrlClassifierDBServiceWorker::AddNoise(const Prefix aPrefix,
                                          const nsCString tableName,
-                                         int32_t aCount,
+                                         uint32_t aCount,
                                          LookupResultArray& results)
 {
   if (aCount < 1) {
@@ -571,7 +573,7 @@ nsUrlClassifierDBServiceWorker::FinishStream()
     // XXX: Only allow forwards from the initial update?
     const nsTArray<ProtocolParser::ForwardedUpdate> &forwards =
       mProtocolParser->Forwards();
-    for (uint32 i = 0; i < forwards.Length(); i++) {
+    for (uint32_t i = 0; i < forwards.Length(); i++) {
       const ProtocolParser::ForwardedUpdate &forward = forwards[i];
       mUpdateObserver->UpdateUrlRequested(forward.url, forward.table, forward.mac);
     }
@@ -984,7 +986,7 @@ nsUrlClassifierLookupCallback::HandleResults()
   // until the next update.
   nsAutoPtr<PrefixArray> cacheMisses(new PrefixArray());
   if (cacheMisses) {
-    for (uint32 i = 0; i < mResults->Length(); i++) {
+    for (uint32_t i = 0; i < mResults->Length(); i++) {
       LookupResult &result = mResults->ElementAt(i);
       if (!result.Confirmed() && !result.mNoise) {
         cacheMisses->AppendElement(result.PrefixHash());
@@ -1129,7 +1131,7 @@ nsUrlClassifierDBService::Init()
   // Should we check document loads for malware URIs?
   nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
 
-  int32_t gethashNoise = 0;
+  uint32_t gethashNoise = 0;
   if (prefs) {
     bool tmpbool;
     rv = prefs->GetBoolPref(CHECK_MALWARE_PREF, &tmpbool);
@@ -1142,9 +1144,10 @@ nsUrlClassifierDBService::Init()
 
     prefs->AddObserver(CHECK_PHISHING_PREF, this, false);
 
-    if (NS_FAILED(prefs->GetIntPref(GETHASH_NOISE_PREF, &gethashNoise))) {
-      gethashNoise = GETHASH_NOISE_DEFAULT;
-    }
+    int32_t tmpint;
+    rv = prefs->GetIntPref(GETHASH_NOISE_PREF, &tmpint);
+    gethashNoise = (NS_SUCCEEDED(rv) && tmpint >= 0) ?
+      static_cast<uint32_t>(tmpint) : GETHASH_NOISE_DEFAULT;
 
     nsXPIDLCString tmpstr;
     if (NS_SUCCEEDED(prefs->GetCharPref(GETHASH_TABLES_PREF, getter_Copies(tmpstr)))) {
@@ -1153,7 +1156,6 @@ nsUrlClassifierDBService::Init()
 
     prefs->AddObserver(GETHASH_TABLES_PREF, this, false);
 
-    int32_t tmpint;
     rv = prefs->GetIntPref(CONFIRM_AGE_PREF, &tmpint);
     PR_ATOMIC_SET(&gFreshnessGuarantee, NS_SUCCEEDED(rv) ? tmpint : CONFIRM_AGE_DEFAULT_SEC);
 
@@ -1261,9 +1263,15 @@ nsUrlClassifierDBService::LookupURI(nsIPrincipal* aPrincipal,
   NS_ENSURE_TRUE(gDbBackgroundThread, NS_ERROR_NOT_INITIALIZED);
   NS_ENSURE_ARG(aPrincipal);
 
+  if (nsContentUtils::IsSystemPrincipal(aPrincipal)) {
+    *didLookup = false;
+    return NS_OK;
+  }
+
   nsCOMPtr<nsIURI> uri;
   nsresult rv = aPrincipal->GetURI(getter_AddRefs(uri));
   NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(uri, NS_ERROR_FAILURE);
 
   uri = NS_GetInnermostURI(uri);
   NS_ENSURE_TRUE(uri, NS_ERROR_FAILURE);
@@ -1506,7 +1514,7 @@ nsUrlClassifierDBService::Shutdown()
     prefs->RemoveObserver(CONFIRM_AGE_PREF, this);
   }
 
-  nsresult rv;
+  DebugOnly<nsresult> rv;
   // First close the db connection.
   if (mWorker) {
     rv = mWorkerProxy->CancelUpdate();
